program Punch;

uses Crt;

VAR Console: text;

(* I/O ERROR CHECKING ----------------------------------------------------- *)

VAR IOErrorCode: word;

FUNCTION IOError: Boolean;
  var Dummy: word;
begin
  if IOErrorCode = 0 then   { save the error code of the first I/O error }
    IOErrorCode := IOResult
  else                      { reset the IOResult function flag }
    Dummy := IOResult;
  IOError := (IOErrorCode <> 0)
  { IOError function remains true until reset by ResetIOErrorProcedure }
end;

PROCEDURE ResetIOError;
begin
  IOErrorCode := 0;                  {enables the following call}
  if IOError then IOErrorCode := 0   {clears IOResult}
end;

FUNCTION IOErrorMessage (FileName: string): string;
  var ErrorNumber: string [5];
begin
  case IOErrorCode of
    2:   IOErrorMessage := 'File ' + FileName + ' not found';
    3:   IOErrorMessage := 'Path not found';
{   4:   IOErrorMessage := 'Too many open files';                }
    5:   IOErrorMessage := 'File access denied (Read-only, directory full ?)';
    100: IOErrorMessage := 'Unexpected end of file';
    101: IOErrorMessage := 'Disk full';
    150: IOErrorMessage := 'Disk is write-protected';
{   151: IOErrorMessage := 'Unknown unit';                       }
    152: IOErrorMessage := 'Drive not ready';
{   153: IOErrorMessage := 'Unknown command';                    }
{   154: IOErrorMessage := 'CRC error in data';                  }
{   155: IOErrorMessage := 'Bad drive request structure length'; }
{   156: IOErrorMessage := 'Disk seek error';                    }
    157: IOErrorMessage := 'Unknown media type';
{   158: IOErrorMessage := 'Sector not found';                   }
    159: IOErrorMessage := 'Printer out of paper';
{   160: IOErrorMessage := 'Device write fault';                 }
{   161: IOErrorMessage := 'Device read fault';                  }
{   162: IOErrorMessage := 'Hardware failure';                   }
    else if IOErrorCode <> 0 then begin
      Str (IOErrorCode, ErrorNumber);
      IOErrorMessage := 'I/O error (Error code ' + ErrorNumber + ')'
    end
    else IOErrorMessage := ''
  end;
  ResetIOError
end;


(* TAPE PUNCH COMMUNICATION ROUTINE (USING LPT1 PORT) --------------------- *)

const
  PrinterData      = $378;
  PrinterControl   = $37A;
  PrinterStatus    = $379;

  MotorOn {AC}     = $01;
  MotorOff         = $09;
  StartStrobe {SC} = $00;
  EndStrobe        = MotorOn;

  AOMask           = $20;  {PE on Centronics port}
  ACMask           = $40;  {ACK on Centronics port}

procedure SwitchMotorOn;
begin
  Port [PrinterControl] := MotorOn
end;

procedure SwitchMotorOff;
begin
  Port [PrinterControl] := MotorOff
end;

function AC: Boolean;
begin
  AC := (Port [PrinterStatus] and ACMask) <> 0
end;

function AO: Boolean;
begin
  AO := (Port [PrinterStatus] and AOMask) <> 0
end;

var TimeoutValue: word;

procedure SetTimeout (Value: word);
begin
  TimeoutValue := Value
end;

function Timeout: Boolean;
begin
  if TimeoutValue = 0 then begin
    Writeln (Console, 'Timeout limit exceeded');
    Halt (1)
  end
  else begin
    Dec (TimeoutValue);
    Delay (1);
    Timeout := false
  end
end;

procedure PunchByte (ByteToPunch: byte);
begin
  SetTimeout (5000);
  repeat until AO or Timeout;
  Port [PrinterData] := ByteToPunch;
  repeat until AC or Timeout;
  Port [PrinterControl] := StartStrobe;
  repeat until not AC or Timeout;
  Port [PrinterControl] := EndStrobe;
end;

procedure PunchBlankTape (NumberOfBlanks: word);
  var i: word;
begin
  for i := 1 to NumberOfBlanks do PunchByte (0)
end;

(* ------------------------------------------------------------------------ *)

procedure GetNumberOfBlanks (Parameter: string; var NumberOfBlanks: integer);
  var ErrorCode: integer;
begin
  Val (Parameter, NumberOfBlanks, ErrorCode);
  if (ErrorCode <> 0) or (NumberOfBlanks < 0) then begin
    Writeln (Console, 'Invalid number of blanks');
    Halt (1)
  end
end;

procedure ReportIOError;
begin
  Writeln (Console, IOErrorMessage (ParamStr (1)));
  Halt (1)
end;

VAR SourceFile: file of byte;
    NumberOfLeadingBlanks, NumberOfTrailingBlanks: integer;
    Checksum: longint;
    ByteToPunch: byte;

{$I-}

BEGIN
  Assign (Console, 'CON');
  Rewrite (Console);

  Writeln (Console, 'DT105S Paper Tape Punch interface program');
  Writeln (Console, '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~');
  Writeln (Console, 'Version 1.02     Petr Horsky, 27. 8. 1988');
  Writeln (Console);
  if ParamCount < 1 then begin
    Writeln (Console, 'Usage:   PUNCH [<path>]<file name> [<leading blanks> [<trailing blanks>]]');
    Writeln (Console);
    Writeln (Console, 'Number of trailing blanks should be specified only if different');
    Writeln (Console, 'from number of leading blanks.   Both the numbers default to 0.');
  end
  else begin
    NumberOfLeadingBlanks := 0;
    NumberOfTrailingBlanks := 0;
    if ParamCount > 1 then begin
      GetNumberOfBlanks (ParamStr (2), NumberOfLeadingBlanks);
      if ParamCount > 2 then
        GetNumberOfBlanks (ParamStr (3), NumberOfTrailingBlanks)
      else
        NumberOfTrailingBlanks := NumberOfLeadingBlanks
    end;

    SwitchMotorOn;

    ResetIOError;
    Assign (SourceFile, ParamStr (1));
    Reset (SourceFile);
    if IOError then ReportIOError;

    PunchBlankTape (NumberOfLeadingBlanks);
    Checksum := 0;
    while not Eof (SourceFile) do begin
      Read (SourceFile, ByteToPunch);
      if IOError then ReportIOError;
      PunchByte (ByteToPunch);
      Checksum := Checksum + ByteToPunch
    end;
    Writeln (Console, 'Byte checksum = ', Checksum);
    PunchBlankTape (NumberOfTrailingBlanks);
    SwitchMotorOff;

    Close (SourceFile);
    if IOError then ReportIOError;
  end
END.
